/*
material
 - oo ui toolkit
*/"use strict"

var prime = require("prime/index"),
	Options = require('prime-util/prime/options'),
	Emitter = require("prime/emitter"),
	mergeObject = require("mout/object/merge"),
	$ = require("elements"),
	zen = require('elements/zen'),
	Component = require('../component');

var Button = prime({

	mixin: [Options, Emitter],

	inherits: Component,

	options: {
		name: 'button',
		type: null, // push, file
		element: {
			tag: 'div'
		}
	},

	/**
	 * Constructor
	 * @param  {Object} options [description]
	 * @return {Object}         [description]
	 */
	constructor: function(options){
		this.setOptions(options);

		this.options = mergeObject(Button.parent.options, this.options);

		this._initElement();
		this._initBinding();

		return this;
	},

	/**
	 * [_initElement description]
	 * @return {[type]} [description]
	 */
	_initElement: function(){
		Button.parent._initElement.call(this);

		var opts = this.options,
			type = opts.type;

		if (type === null)
			type = 'icon-text';

		if ((opts.icon && type.indexOf('icon') > -1) || type === 'file')
			this._initIcon(type);

		this.addClass('ui-button');

		if (opts.type === 'file') {
			this._initFile(type);
		}

		if (type === 'submit' || type === 'reset')
			this.setAttribute('type', type);
	},

	/**
	 * [_initIcon description]
	 * @param  {[type]} type [description]
	 * @return {[type]}      [description]
	 */
	_initIcon: function(type) {
		//console.log('_initIcon', type, this.element);
		var opts = this.options;

		var tag = 'span';
		if (type === 'file')	tag = 'label';

		var pos = 'top';
		if (type === 'text-icon')
			pos = 'bottom';

		this.icon = new component({
			tag: 'i',
			'class': 'button-icon',
			for: 'upload'
		}).inject(this.element, pos);		

		this.icon.addClass('fa');
		this.icon.addClass(opts.icon);
		console.log(opts.icon);
	},

	/**
	 * [_initFile description]
	 * @param  {[type]} type [description]
	 * @return {[type]}      [description]
	 */
	_initFile: function(type) {
		var self = this;

		var file = new component({
			tag: 'input',
			type: 'file',
			name: 'upload',
			id: 'upload'
		}).inject(this.element);

		file.on('change', function(info) {
			console.log('change', info);
		});

		file.onchange = function(info) {
			var files = this.files;
			console.log(files);
			if (files)
			self.emit('uploadFile', [files]);
		};
	},

	/**
	 * [_initEvents description]
	 * @return {[type]} [description]
	 */
	_initBinding: function(){
		var self = this,
			opts = this.options,
			state = opts.state;

		this.element.on('click', function(e) {
			console.log('click', e.relativeX);
			e.stopPropagation();
			if (opts.emit && self.state !== 'disabled')
				self.emit(opts.emit);

			self.emit('click', e);
		});
	}
});


module.exports = Button;
