
/**
* Minimalistic base of all ui components.
*
* @class ui.component
* @requires primish
* @return {parent} Class
* @example (start code)	new ui.component(object); (end)
* @author [politan]
* @copyright © 1999-2014 - Jerome D. Vial. All Rights reserved.
*/
ui.field = primish({

	extend: ui.component,
	//implement: [options, emitter],
	name: 'ui-field',

	options: {
		type: 'text',
		element: {
			tag: 'span'
		},
		label: {},
		input: {}
	},

	/**
	 * Create a div and a hidden input to receive the selected value
	 * @function _initElement
	 * @extends {_initElement}
	 * @return {[type]} [description]
	 */
	_initElement: function(){
		//console.log('_initElement', this.name);
		//create a new div as input element
		this.parent('_initElement');

		var opts = this.options;

		if (this.options.label)
			this._initLabel();

		this._initInput();
	},

	/**
	 * [_initLabel description]
	 * @return {[type]} [description]
	 */
	_initLabel: function()  {
		var text = this.options.name;

		if (this.options.useTextAsLabel)
			text = this.options.text;

		this.label = new ui.component({
			tag: 'label',
			text: this.options.name,
			'for': this.options.name
		}).inject(this.element);

	},

	/**
	 * [_initInput description]
	 * @return {[type]} [description]
	 */
	_initInput: function()  {
		var self = this,
			opts = this.options,
			name = opts.name,
			type = opts.type || 'text';

		//console.log('input option', this.options);

		var input = this.input = new ui.component({
			tag: 'input',
			type: this.options.type,
			text: this.options.name,
			autocapitalize: 'off',
            autocorrect: 'off'
		}).inject(this.element);

		Dom.onFocus(input.element, function(e) {
			//console.log('focus', e);
			self.trigger('focus');
		});

		Dom.onBlur(input.element, function(e) {
			//console.log('blur', e);
			self.trigger('blur');
		});

		Dom.onChange(input.element, function(e) {
			//console.log('change', e);
			//self.trigger('focus');
		});

		this.input.setAttribute('type', type);

		if (name)
			this.input.setAttribute('name', name);

	},

	/**
	 * [_initName description]
	 * @param  {[type]} name [description]
	 * @return {[type]}      [description]
	 */
	_initName: function(name) {
		var opts = this.options;

		if (opts.name) {
			this.label.set('html', name);
			this.input.set('name', name);
		}
	},

	_initValue: function(){
		var opts = this.options;

		if (opts.value)
			this.setValue(opts.value);
	},

	focus: function() {
		this.input.element.focus();
	},

	getValue: function(){
		//console.log('getValue', this.input);
		return this.input.element.value;
	},

	setValue: function(value){
		this.input.set('value', value);
		this.value = value;
		this.fireEvent('change' , value);
	},

	_initEvents: function() {
		var self = this;
		this.parent('_initEvents');

		this.on('blur', function() {
			self.setState(null);
		});

		this.on('focus', function() {
			self.setState('focus');
		});
	},

	set: function(name, value) {


	}
});
